@@ -99,7 +99,7 @@ static GtkActionEntry edit_metadata_action_entries[] = {
 
 	{ "Edit_Metadata", GTK_STOCK_EDIT,
 	  N_("Comment"), "<control>M",
-	  N_("Edit the comment an other information of the selected files"),
+	  N_("Edit the comment and other information of the selected files"),
 	  G_CALLBACK (gth_browser_activate_action_edit_metadata) },
 
         { "Edit_QuickTagOther", NULL,
@@ -283,7 +283,7 @@ insert_tag_menu_item (BrowserData *data,
 	GtkWidget *image;
 
 	item = gtk_image_menu_item_new_with_label (tag);
-	/*gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item), TRUE);*/
+	gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item), TRUE);
 
 	image = gtk_image_new_from_icon_name ("tag", GTK_ICON_SIZE_MENU);
 	gtk_widget_show (image);
