@@ -811,7 +811,7 @@ gfile_get_relative_path (GFile *file,
 /* construct a GFile for a GthWebExporter */
 
 
-GFile *
+static GFile *
 get_album_file (GthWebExporter *self,
 		GFile          *target_dir,
 		const char     *subdir,
@@ -824,7 +824,7 @@ get_album_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_html_index_dir (GthWebExporter *self,
 		    const int       page,
 		    GFile          *target_dir)
@@ -836,7 +836,7 @@ get_html_index_dir (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_html_image_dir (GthWebExporter *self,
 		    GFile          *target_dir)
 {
@@ -844,7 +844,7 @@ get_html_image_dir (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_theme_file (GthWebExporter *self,
 		GFile          *target_dir,
 		const char     *filename)
@@ -853,7 +853,7 @@ get_theme_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_html_index_file (GthWebExporter *self,
 		     const int       page,
 		     GFile          *target_dir)
@@ -876,7 +876,7 @@ get_html_index_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_html_image_file (GthWebExporter *self,
 		     ImageData      *image_data,
 		     GFile          *target_dir)
@@ -893,7 +893,7 @@ get_html_image_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_thumbnail_file (GthWebExporter *self,
 		    ImageData      *image_data,
 		    GFile          *target_dir)
@@ -910,7 +910,7 @@ get_thumbnail_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_image_file (GthWebExporter *self,
 		ImageData      *image_data,
 		GFile          *target_dir)
@@ -926,7 +926,7 @@ get_image_file (GthWebExporter *self,
 }
 
 
-GFile *
+static GFile *
 get_preview_file (GthWebExporter *self,
 		  ImageData      *image_data,
 		  GFile          *target_dir)
@@ -2365,6 +2365,31 @@ save_resized_image_ready_cd (GthFileData *file_data,
 }
 
 
+static const char *
+get_format_description (const char *mime_type)
+{
+	const char *description = NULL;
+	GSList     *formats;
+	GSList     *scan;
+
+	formats = gdk_pixbuf_get_formats ();
+	for (scan = formats; ! description && scan; scan = scan->next) {
+		GdkPixbufFormat  *format = scan->data;
+		char            **mime_types;
+		int               i;
+
+		mime_types = gdk_pixbuf_format_get_mime_types (format);
+		for (i = 0; ! description && mime_types[i] != NULL; i++)
+			if (g_strcmp0 (mime_types[i], mime_type) == 0)
+				description = gdk_pixbuf_format_get_description (format);
+	}
+
+	g_slist_free (formats);
+
+	return description;
+}
+
+
 static gboolean
 save_resized_image (gpointer data)
 {
@@ -2378,6 +2403,8 @@ save_resized_image (gpointer data)
 
 	image_data = self->priv->current_file->data;
 	if (self->priv->copy_images && (image_data->image != NULL)) {
+		char        *filename_no_ext;
+		char        *size;
 		GFile       *destination;
 		GthFileData *file_data;
 
@@ -2387,6 +2414,29 @@ save_resized_image (gpointer data)
 				   FALSE,
 				   (double) (self->priv->image + 1) / (self->priv->n_images + 1));
 
+		/* change the file extension to jpeg */
+
+		filename_no_ext = _g_uri_remove_extension (image_data->dest_filename);
+		g_free (image_data->dest_filename);
+		image_data->dest_filename = g_strconcat(filename_no_ext, ".jpeg", NULL);
+		g_free (filename_no_ext);
+
+		/* change the file type */
+
+		gth_file_data_set_mime_type (image_data->file_data, "image/jpeg");
+		g_file_info_set_attribute_string (image_data->file_data->info, "general::format", get_format_description ("image/jpeg"));
+
+		/* change the image dimensions info */
+
+		g_file_info_set_attribute_int32 (image_data->file_data->info, "image::width", image_data->image_width);
+		g_file_info_set_attribute_int32 (image_data->file_data->info, "image::height", image_data->image_height);
+		g_file_info_set_attribute_int32 (image_data->file_data->info, "frame::width", image_data->image_width);
+		g_file_info_set_attribute_int32 (image_data->file_data->info, "frame::height", image_data->image_height);
+		size = g_strdup_printf (_("%d × %d"), image_data->image_width, image_data->image_height);
+		g_file_info_set_attribute_string (image_data->file_data->info, "general::dimensions", size);
+
+		/* save the pixbuf */
+
 		destination = get_image_file (self, image_data, self->priv->tmp_dir);
 		file_data = gth_file_data_new (destination, NULL);
 		_gdk_pixbuf_save_async (image_data->image,
@@ -2554,16 +2604,35 @@ create_squared_thumbnail (GdkPixbuf     *p,
 }
 
 
-static void
-image_loader_ready_cb (GthImageLoader *iloader,
-		       GError         *error,
-		       gpointer        data)
+static int
+image_data_cmp (gconstpointer a,
+		gconstpointer b,
+		gpointer      user_data)
 {
-	GthWebExporter *self = data;
+	GthWebExporter *self = user_data;
+	ImageData      *idata_a = (ImageData *) a;
+	ImageData      *idata_b = (ImageData *) b;
+
+	return self->priv->sort_type->cmp_func (idata_a->file_data, idata_b->file_data);
+}
+
+
+static void
+image_loader_ready_cb (GObject      *source_object,
+                       GAsyncResult *result,
+                       gpointer      user_data)
+{
+	GthWebExporter *self = user_data;
 	ImageData      *idata;
 	GdkPixbuf      *pixbuf;
 
-	if (error != NULL) {
+	if (! gth_image_loader_load_image_finish (GTH_IMAGE_LOADER (source_object),
+						  result,
+						  &pixbuf,
+						  NULL,
+						  NULL,
+						  NULL))
+	{
 		load_next_file (self);
 		return;
 	}
@@ -2572,7 +2641,7 @@ image_loader_ready_cb (GthImageLoader *iloader,
 
 	/* image */
 
-	idata->image = pixbuf = g_object_ref (gth_image_loader_get_pixbuf (iloader));
+	idata->image = g_object_ref (pixbuf);
 	if (self->priv->copy_images && self->priv->resize_images) {
 		int w = gdk_pixbuf_get_width (pixbuf);
 		int h = gdk_pixbuf_get_height (pixbuf);
@@ -2594,7 +2663,7 @@ image_loader_ready_cb (GthImageLoader *iloader,
 
 	/* preview */
 
-	idata->preview = pixbuf = g_object_ref (gth_image_loader_get_pixbuf (iloader));
+	idata->preview = g_object_ref (pixbuf);
 	if ((self->priv->preview_max_width > 0) && (self->priv->preview_max_height > 0)) {
 		int w = gdk_pixbuf_get_width (pixbuf);
 		int h = gdk_pixbuf_get_height (pixbuf);
@@ -2626,7 +2695,7 @@ image_loader_ready_cb (GthImageLoader *iloader,
 
 	/* thumbnail. */
 
-	idata->thumb = pixbuf = gth_image_loader_get_pixbuf (iloader);
+	idata->thumb = g_object_ref (pixbuf);
 	g_object_ref (idata->thumb);
 
 	if ((self->priv->thumb_width > 0) && (self->priv->thumb_height > 0)) {
@@ -2691,19 +2760,8 @@ image_loader_ready_cb (GthImageLoader *iloader,
 	}
 	else
 		self->priv->saving_timeout = g_idle_add (save_image_preview, self);
-}
 
-
-static int
-image_data_cmp (gconstpointer a,
-		gconstpointer b,
-		gpointer      user_data)
-{
-	GthWebExporter *self = user_data;
-	ImageData      *idata_a = (ImageData *) a;
-	ImageData      *idata_b = (ImageData *) b;
-
-	return self->priv->sort_type->cmp_func (idata_a->file_data, idata_b->file_data);
+	g_object_unref (pixbuf);
 }
 
 
@@ -2727,8 +2785,14 @@ load_current_file (GthWebExporter *self)
 			   g_file_info_get_display_name (file_data->info),
 			   FALSE,
 			   (double) (self->priv->image + 1) / (self->priv->n_images + 1));
-	gth_image_loader_set_file_data (self->priv->iloader, file_data);
-	gth_image_loader_load (self->priv->iloader);
+
+	gth_image_loader_load (self->priv->iloader,
+			       file_data,
+			       -1,
+			       G_PRIORITY_DEFAULT,
+			       gth_task_get_cancellable (GTH_TASK (self)),
+			       image_loader_ready_cb,
+			       self);
 }
 
 
@@ -3186,13 +3250,7 @@ gth_web_exporter_init (GthWebExporter *self)
 	self->priv->file_list = NULL;
 	self->priv->tmp_dir = NULL;
 	self->priv->interrupted = FALSE;
-
-	self->priv->iloader = gth_image_loader_new (FALSE);
-	g_signal_connect (G_OBJECT (self->priv->iloader),
-			  "ready",
-			  G_CALLBACK (image_loader_ready_cb),
-			  self);
-
+	self->priv->iloader = gth_image_loader_new (NULL, NULL);
 	self->priv->error = NULL;
 }
 
